#! /usr/bin/python3

import datetime

def daterange(start, stop, step=datetime.timedelta(days=1), inclusive=False):
    """ Generator for dates from start to stop in increments of step

    Like range, excludes the stop date, but inclusive=True changes this behavior.

    >>> from csppfetch.daterange import daterange
    >>> from datetime import date,time,datetime,timedelta
    >>> list(daterange(date(2123,4,5), date(2123,4,7)))
    [datetime.date(2123, 4, 5), datetime.date(2123, 4, 6)]
    >>> list(daterange(date(2123,4,5), date(2123,4,7), inclusive=True))
    [datetime.date(2123, 4, 5), datetime.date(2123, 4, 6), datetime.date(2123, 4, 7)]
    >>> list(daterange(date(2000,1,2), date(2000,1,1)))
    []
    >>> list(daterange(datetime(2000,1,1,12), datetime(1999,12,31,12), timedelta(hours=-11)))
    [datetime.datetime(2000, 1, 1, 12, 0), datetime.datetime(2000, 1, 1, 1, 0), datetime.datetime(1999, 12, 31, 14, 0)]
    >>> list(daterange(date(2000,1,2), date(2000,1,1), timedelta()))
    Traceback (most recent call last):
    ...
    ValueError: daterange() arg 3 must not be a datetime.timedelta of 0
    """
    if step == datetime.timedelta():
        raise ValueError("daterange() arg 3 must not be a datetime.timedelta of 0")

    nextdate = start

    import operator
    test = operator.lt
    if step < datetime.timedelta():
        test = operator.gt

    while test(nextdate, stop):
        yield nextdate
        nextdate += step
    if inclusive and nextdate == stop:
        yield nextdate
